''' Bugzilla Remote Interface '''

import cgi
import os
import pycurl
import re
import urllib

from StringIO import StringIO

DEBUG=1

class BugzillaLoginError(Exception):
    pass

class BugzillaUploadError(Exception):
    pass

class BugzillaAdapter:

    def __init__(self, tmpdir):
        self.tmpdir = tmpdir
        self.cookiejar = os.path.join(self.tmpdir, 'cookiejar')

    def findall_in_url(self, pattern, url, opts=[]):
        headers, response = self.get_response(url, opts)
        regexp = re.compile(pattern)
        return regexp.findall(response)

    def is_login_required(self, url):
        pattern = r'<.*?form.*?name="login".*?>'
        return bool(self.findall_in_url(pattern, url))

    def login(self, url, username, password):
        formdata = [
            ('action', 'enter'),
            ('Bugzilla_login', username),
            ('Bugzilla_password', password),
            ('GoAheadAndLogIn', 'Log in')]

        headers, response = self.get_response(url,
                                              opts=[(pycurl.HTTPPOST, formdata)])

        with open(self.cookiejar, 'r+b') as fd:
            if not 'Bugzilla_login' in fd.read():
                pattern = re.compile(r'<title>(.*?)</title>')
                matches = pattern.search(response).groups()
                reason = matches[0]
                raise BugzillaLoginError(reason)

        return username

    def get_response(self, url, opts=[]):
        curl = pycurl.Curl()
        headers = StringIO()
        response = StringIO()

        curl.setopt(pycurl.URL, url)
        curl.setopt(pycurl.COOKIEFILE, self.cookiejar)
        curl.setopt(pycurl.COOKIEJAR, self.cookiejar)
        curl.setopt(pycurl.FOLLOWLOCATION, 1)
        curl.setopt(pycurl.HEADERFUNCTION, headers.write)
        curl.setopt(pycurl.WRITEFUNCTION, response.write)
        curl.setopt(pycurl.SSL_VERIFYPEER, False)

        for key, value in opts:
            curl.setopt(key, value)

        curl.perform()
        curl.close()

        return headers.getvalue(), response.getvalue()

    def lookup_registered_email(self, emails):
        ''' Returns first email in list of emails found to exist in remote bugzilla '''
        pattern = re.compile(r'<span class="bz_result_count">(\d+) .* found')
        for email in emails:
            params = {
                'email1': email,
                }
            params_enc = urllib.urlencode(params)

            url = "%s?emailcc1=1&emailreporter1=1&emaillongdesc1=1&emailtype1=substring&query_format=advanced&emailassigned_to1=1&emailqa_contact1=1&bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED&bug_status=UNCONFIRMED&bug_status=RESOLVED&bug_status=VERIFIED&bug_status=CLOSED&bug_status=NEEDINFO&bug_status=PLEASETEST&%s" %(self.bzurl, params_enc)

            headers, response = self.get_response(url)
            result = pattern.search(response)
            if result:
                matches = result.groups()
                if len(matches)>0:
                    if DEBUG:
                        print "FOUND %s in Bugzilla in %d bug reports" %(email, int(matches[0]))
                    return email
            elif DEBUG:
                print "%s not registered with Bugzilla" %(email)

        # No email was found
        return None

    def upload_attachment(self, attachment, url, bugid):
        tmpfile = os.path.join(self.tmpdir, attachment.title)

        pattern = r'<input\s*type="hidden"\s*name="(.*?)"\s*value="(.*?)"'
        formdata = self.findall_in_url(pattern, url)

        formdata.extend([
            ('data', (pycurl.FORM_FILE, tmpfile)),
            ('description', attachment.title),
            ('ispatch', attachment.is_patch() and '1' or '0')])

        if attachment.content_type:
            formdata.extend([
                ('contenttypemethod', 'manual'),
                ('contenttypeentry', attachment.content_type)])
        else:
            formdata.extend([
                ('contenttypemethod', 'autodetect')])

        headers, response = self.get_response(url,
                                        opts=[(pycurl.HTTPPOST, formdata),])
        title = re.escape(cgi.escape(attachment.title))
        pattern = re.compile(
            r'<dt>.*<a.*title="%s".*'
            r'<a.*href="show_bug\.cgi\?id=%s.*</dt>' %
            (title, bugid),
            re.DOTALL)
        if not pattern.search(response):
            pattern = re.compile(r'<title>(.*?)</title>')
            matches = pattern.search(response).groups()
            reason = matches[0]
            raise BugzillaUploadError(reason)

        return True


