#!/usr/bin/python

# Purpose of script is to close all bugs filed using checkbox with
# a friendly message, since they largely lack the information we
# need to troubleshoot them.

from arsenal_lib import *
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

dryrun      = False
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

message = """
Thank you for seeking to improve Ubuntu.  It seems you have filed a bug
against X using the new System Testing utility. Unfortunately, this tool
is filing bugs without collecting sufficient information to properly
diagnose the issues.  I wish we could reply to every report to guide you
to gather the correct information, but it happens that so many people
are using this testing feature to file bug reports that we simply cannot
keep up with the influx.  I'm replying in bulk so I can get everyone 
pointed into a direction most likely to result in getting you a useful
workaround and us a complete bug report that can be solved.

Most of these problems describe issues that have documented
troubleshooting procedures, so perhaps the most helpful thing I can
do is point you to them:

  http://wiki.ubuntu.com/X/Troubleshooting

These guides explain why things break the way they do, and in some cases
provide workarounds.

I would ask that you first go through the appropriate guide for your
problem, particularly if you are having problems with resolution,
crashes, fonts, or blank screen issues, and then file a bug report using
the following command:

   ubuntu-bug xorg

This collects more detailed information than the System Testing tool
does.  Please also write a detailed explanation, indicating when you 
first discovered the problem, analysis steps you've done, workarounds
that work, and so on.

Or, if you suspect the problem has something to do with the way the test
script itself is working (such as if Ubuntu worked fine until you ran
the script), then file the bug against the test suite:

   ubuntu-bug checkbox

Thanks again, and good luck!
"""

for source_pkg in source_pkgs:
    subject = "Your System Testing bug report"

    s = d.getSourcePackage(name = source_pkg)
    for bugtask in s.searchTasks(tags="checkbox-bug"):
        bug = ArsenalBug(bugtask.bug, lp.launchpad)

        print "Set from " + bugtask.status + " to Invalid: " + bug.title
        if not dryrun:
            bug.append_comment(message)
            bugtask.transitionToStatus(status = "Invalid")
