#!/usr/bin/python

from arsenal_lib import *
from lpltk import LaunchpadService

source_pkg  = "xserver-xorg-video-intel"
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = False

message = """
We have a PPA with some new tools for debugging X freezes:

  https://launchpad.net/~ubuntu-x-swat/+archive/x-freeze-test

I know how irritating X freezes can be.  They're also typically
quite hard to debug, but the information provided by these new
tools should help upstream figure them out.

You can help by doing the following:

 A. Install the PPA packages on Jaunty and boot kernel 2.6.30-rc2
 B. Reproduce your freeze
 C. ssh into the machine and run the steps to collect the info
 D. Attach the tarball of the results to this bug report

With this information, we'll be able to forward your bug upstream.

(For more information on triaging X freeze bugs, see
 https://wiki.ubuntu.com/X/Troubleshooting/Freeze )
"""

def already_has_xfreeze_ppa(bug):
    regex_ppa = re.compile('x-freeze-test', re.IGNORECASE)
    
    # Check messages
    for m in bug.messages:
        if regex_ppa.search(m.content):
            return True

    return False

def already_has_gpu_file(bug):
    regex_gpu = re.compile('(gpu|dri).*\.tgz$', re.IGNORECASE)
    regex_tarball = re.compile('(tar|tar\.gz|tgz|tar\.bz2)$', re.IGNORECASE)
    regex_failsafe = re.compile('^failsafe', re.IGNORECASE)
    
    # Check attachments 
    for a in bug.attachments:
        if regex_gpu.search(a.title):
            return True
        if regex_tarball.search(a.title):
            if regex_failsafe.search(a.title):
                continue
            return True

    return False

regex_freeze = re.compile('freeze', re.IGNORECASE)
regex_8xx    = re.compile('\[i8', re.IGNORECASE)
count = 0
for bugtask in d.searchTasks(search_text="freeze"):
    if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
        continue

    bug = ArsenalBug(bugtask.bug, lp.launchpad)

    # Verify 'freeze' is in the subject line
    if not regex_freeze.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[!FREEZE]", bugtask.status, bug.title)
        continue

    # Skip i8xx bugs
    if regex_8xx.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[8XX]", bugtask.status, bug.title)
        continue

    if bugtask.status == "In Progress" or bugtask.status == "Fix Committed":
        print "%-8s %-10s %-12s %s" % (bug.id, "[INPROG]", bugtask.status, bug.title)
        continue

    # Check all comments to see if we've already asked for the info
    if already_has_xfreeze_ppa(bugtask.bug):
        print "%-8s %-10s %-12s %s" % (bug.id, "[KNOWS]", bugtask.status, bug.title)
        continue

    # Check filenames to see if gpu info is already attached
    if already_has_gpu_file(bugtask.bug):
        print "%-8s %-10s %-12s %s" % (bug.id, "[GPU-DUMP]", bugtask.status, bug.title)
        continue

    if not dryrun:
        bug.append_comment(message)
        bugtask.transitionToStatus(status = "New")
        bugtask.transitionToStatus(status = "Incomplete")
    count = count + 1
    print "%-8s %-10s %-12s %s" % (bug.id, "-freeze-", bugtask.status, bug.title)


print count, " bugs changed for ", source_pkg

