#!/usr/bin/python

# Purpose of script is to tag/close/comment all bugs in a given LRM
# package with friendly comment about the change.

from arsenal_lib import *
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

message = """
[This is an automated message]

As of Intrepid (8.10), we have a dedicated package 'fglrx-installer' for fglrx bugs, which now includes a process for upstreaming bugs to AMD.

  http://bugs.launchpad.net/ubuntu/+source/fglrx-installer

To transition your bug into the new fglrx-package, we need your help.  Please do the following:

 a.  Verify the bug occurs in Intrepid.
     (Intrepid ISOs: http://cdimage.ubuntu.com)
 b.  If you haven't already, please include in the bug:
     * Your /var/log/Xorg.0.log
     * The output of `lspci -vvnn`
     * Steps to reproduce the issue
 c.  Click 'Also affects distribution'
 d.  Set 'Source Package Name' to 'fglrx-installer'
 e.  Click Continue

Thank you.  This will assist us in reviewing and upstreaming your fglrx bug, as appropriate.

[We'll expire the fglrx bugs in l-r-m-* in a month or so.]
"""

def is_fglrx_bug(title):
    re_fglrx = re.compile("(fglrx|flgrx| ATI |radeon|aticonfig|atievent)",  re.IGNORECASE)
    return re_fglrx.search(title)

for pkg in source_pkgs:
    for bugtask in d.searchTasks(search_text=pkg):
        if bugtask.bug_target_display_name != pkg + " (Ubuntu)":
            continue

        bug = ArsenalBug(bugtask.bug, lp.launchpad)

        if is_fglrx_bug(bug.title):
            print bug.id, "  ", bugtask.status, "  ", bug.title
            bug.append_tags("fglrx")
            bug.append_comment(message)

            print "  ----> Requested transitioning"

# Tag?
# Close as won't fix?
#  Close 2.6.22 tasks
#  2.6.24 tasks maybe leave open in case of SRU's?

