#!/usr/bin/python

# Purpose of script is to tag/close/comment all bugs in the nvidia-180
# package with friendly comment about a change.

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltk import LaunchpadService


source_pkg  = 'nvidia-graphics-drivers-180'
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

message_new_nvidia = """
[This is an automated message]

In Jaunty (9.04), we have just updated to the latest
nvidia-graphics-drivers-180 package from nVidia, version 180.44.

This package provides fixes for a large number of bugs, and we need your
assistance in testing if it fixes the issue you reported.

To do this, please do the following:

 a.  Update to the 180.44 version of -nvidia using your favorite update
     method

 b.  Attempt to reproduce your bug

 c.  If your bug still remains, please simply reply to this email
     indicating so.

 d.  If your bug is now solved, you can help us by setting your bug
     report to Fix Released:
     * In launchpad, go to your bug report
     * Click on the downward pointing arrow under Status
     * Set the Status field to 'Fix Released'
     * Comment on the change, such as, 'Verified fixed in 180.44'
     * Click 'Save Changes'

 e.  If the original problem is solved but there are now other problems,
     please close the original bug and open new ones for those issues.
 
Thank you!

For details on the changes in this version of -nvidia, please see:

   http://www.nvidia.com/object/linux_display_ia32_180.44.html
"""

message_dropped_177 = """
[This is an automated message]

In Jaunty (9.04) we have dropped the nvidia-graphics-driver-177 package, as
it is superseded by nvidia-graphics-drivers-180.

Your reported issue may still be a problem in Intrepid (8.10) however
fixing it would require an update to the binary driver, which is not
permitted under our stable release update policy.  (Because the driver is
binary-only, we cannot examine the source code for regression potential,
so must assume every binary driver update carries high risk of regression
for users not experiencing your issue.)

Because of this, we are closing this bug as expired.

You are encouraged to file your -nvidia bug reports against the
nvidia-graphics-drivers-180 package.
"""

message = message_new_nvidia

for bugtask in d.searchTasks(search_text=source_pkg):
    if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
        continue

    tries = 5
    while tries > 0:
        tries -= 1
        try:
            bug = ArsenalBug(bugtask.bug, lp.launchpad)

            print bug.id, "  ", bugtask.status, "  ", bug.title
            bug.append_comment(message)

#                bugtask.transitionToStatus(status = "Invalid")
#                print "  ----> Closed bug"
            tries = 0

        except HTTPError, e:
            print "*** Exception *** " + e.content
            print "*** Trying ",bug.id," again..."
            print
            raise
