#!/usr/bin/python

### Customization Parameters ###
source_pkg  = "xserver-xorg-video-intel"
release     = "Lucid"
test_target = "ppa"  # "ppa" or "iso"
ppa_url     = "https://edge.launchpad.net/~apw/+archive/red"

description = """
"""

opt_auto_notice     = True    # Admit that this is an automatic notification

opt_retest_new          = True    # status New
opt_retest_incomplete   = True    # status Incomplete
opt_retest_inprogress   = False   # status In Progress
opt_retest_fixcommitted = False   # status Fix Committed
opt_retest_upstreamed   = False   # Has upstream bug watch
opt_retest_needs        = False   # subject mentions Needs something...
opt_retest_quirk        = True    # Looks like just needs a quirk
opt_retest_workflow     = False   # Looks like a workflow bug report
opt_retest_master       = False   # Looks like a master bug not worth retesting
opt_retest_wishlist     = False   # importance Wishlist
opt_retest_assigned     = True    # is assigned to someone
opt_retest_patched      = False   # has a patch attached
opt_retest_asked        = False   # has been asked to retest before
opt_retest_devel        = True    # mentions current devel release in title/desc/comments
opt_retest_edgers       = False   # mentions xorg-edgers

# Add regexes to skip, like specific version #'s, ppa names, etc.
exclude_regexes     = ( )

# Add tags to skip
exclude_tags        = ("kubuntu", "ppc", "omit")

# Set dry_run to False to make script actually perform the changes
dry_run = False

### End of Customization Parameters ###

from arsenal_lib import *
from lpltk import LaunchpadService

lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
fmt         = "%-8s %-10s %-12s %s"

re_quirk    = re.compile("quirk", re.IGNORECASE)
re_workflow = re.compile("^Please ", re.IGNORECASE)
re_master   = re.compile("^MASTER ")
re_needs    = re.compile("Needs")

def main():
    count       = 0
    
    driver      = driver_nickname(source_pkg)
    message     = generate_message(driver)

    if dry_run:
        print message
        print "-----------------------------------------------\n"

    s = d.getSourcePackage(name = source_pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)

        exclusion_reason = check_exclusion_criteria(bug, bugtask)
        if exclusion_reason:
            print fmt % (bug.id, "["+exclusion_reason+"]", bugtask.status, bug.title)
            continue
    
        if not dry_run:
            if bug.append_comment(message):
                if bugtask.status == "Incomplete":
                    bugtask.transitionToStatus(status = "New")
                bugtask.transitionToStatus(status = 'Incomplete')

        print fmt % (bug.id, "-retest-", bugtask.status, bug.title)
        count = count + 1

    if count > 0:
        if dry_run:
            print count, " bugs would be changed for ", source_pkg
        else:
            print count, " bugs changed for ", source_pkg

def check_exclusion_criteria(bug, bugtask):
    if opt_retest_new == False and bugtask.status == "New":
        return bugtask.status.upper()

    if opt_retest_incomplete == False and bugtask.status == "Incomplete":
        return bugtask.status.upper()

    if opt_retest_inprogress == False and bugtask.status == "In Progress":
        return bugtask.status.upper()

    if opt_retest_fixcommitted == False and bugtask.status == "Fix Committed":
        return bugtask.status.upper()

    if opt_retest_upstreamed == False and bug.has_upstream_task():
        return "UPSTREAM"

    if opt_retest_needs == False and re_needs.search(bug.title):
        return "NEEDS"

    if opt_retest_quirk == False and re_quirk.search(bug.title):
        return "QUIRK"

    if opt_retest_master == False and re_master.search(bug.title):
        return "MASTER"

    if opt_retest_workflow == False and re_workflow.search(bug.title):
        return "WORKFLOW"

    if opt_retest_wishlist == False and bugtask.importance == "Wishlist":
        return "WISHLIST"

    if opt_retest_assigned == False and bugtask.assignee:
        return "ASSIGNED"

    if opt_retest_patched == False and bug.has_patch():
        return "PATCH"

    if opt_retest_asked == False and bug.has_text_match("A new version of .* driver is now available"):
        return "ASKED"

    for regex in exclude_regexes:
        print "Checking exclude_regexes"
        if bug.has_text_match(regex):
            return "REGEX-MATCH"

    for tag in exclude_tags:
        if bug.has_tag(tag):
            return tag.upper()

    return None

def generate_message(driver):
    ### Admit we're a bot ###
    if opt_auto_notice:
        message = "[This is an automatic notification.]\n"

    ### Main message to convey ###
    message += """
A new version of the %s driver is now available in %s.
""" % (driver, release)

    ### Detailed Explanation ###
    message += description

    ### Testing directions ###
    if test_target == "iso":
        message += """
Would you mind testing current %s and seeing if it is still a
problem?  CD LiveCD ISO images are available here:

  http://cdimages.ubuntu.com/releases/
""" % (release)

    elif test_target == "ppa":
        message += """
I've posted the new version of this driver to the following PPA,
would you mind testing it and seeing if it resolves the bug you
reported?

  %s

If you're not running this release of Ubuntu, you can try booting the %s
LiveCD and loading the PPA onto it, and then log out/in to restart X.
ISOs are available at http://cdimages.ubuntu.com/releases/
""" % (ppa_url, release)

    ### Directions ###
    message += """
After testing %s, report back here whether it's still an issue or not,
and if it is please post a fresh Xorg.0.log and 'dmesg' output.

Note there could be new bugs... please file these as new reports using
the command 'ubuntu-bug xorg'.
""" % (release)

    return message

def driver_nickname(source_pkg):
    re_nvidia = re.compile("nvidia-graphics-drivers-(.*)")
    re_xorg = re.compile("xserver-xorg-video(-.*)")
    
    if source_pkg == "fglrx-installer":
        return "-fglrx"
    elif re_nvidia.search(source_pkg):
        return "-nvidia"
    else:
        m = re_xorg.match(source_pkg)
        return m.group(1)

main()
