#!/usr/bin/python

# This is a hackable script for requesting retests of things
tags        = ["-maverick", "-meerkat", "-lucid", "-kubuntu", "-ppc", "-omit"]
dry_run     = False

from arsenal_lib import *
from lpltk import LaunchpadService

source_pkg  = sys.argv[1]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

re_workflow = re.compile("^Please ", re.IGNORECASE)
re_master   = re.compile("^MASTER ")
re_needs    = re.compile("Needs")


message_livecd = """[This is an automatic notification.]

A new version of the intel driver is now available in lucid.

Would you mind testing current %s and seeing if it is still a
problem?  CD LiveCD ISO images are available here:

  http://cdimages.ubuntu.com/releases/

Note there could be new bugs... please file these as new reports using
the command 'ubuntu-bug xorg'.
"""

message_ppa = """[This is an automatic notification.]

I've posted the new version of this driver to the following PPA,
would you mind testing it and seeing if it resolves the bug you
reported?

    https://edge.launchpad.net/~apw/+archive/red

If you're not running this release of Ubuntu, you can try booting the lucid
LiveCD and loading the PPA onto it, and then log out/in to restart X.
ISOs are available at http://cdimages.ubuntu.com/releases/

Once you've done the testing, please update your bug report by running
the command:

  apport-collect [bug-number]

Note there could be new bugs... please file these as new reports using
the command 'ubuntu-bug xorg'.
"""

message_pre_lucid = """[This is an automatic notification.]

Hi REPORTER,

This bug was reported against an earlier version of Ubuntu, can you
test if it still occurs on Lucid?

Please note we also provide technical support for older versions of
Ubuntu, but not in the bug tracker.  Instead, to raise the issue through
normal support channels, please see:

    http://www.ubuntu.com/support


If you are the original reporter and can still reproduce the issue on
Lucid, please run the following command to refresh the report:

  apport-collect BUGNUMBER

If you are not the original reporter, please file a new bug report, so
we can work with you as the original reporter instead (you can reference
bug BUGNUMBER in your report if you think it may be related):

  ubuntu-bug xorg

If by chance you can no longer reproduce the issue on Lucid or if you
feel it is no longer relevant, please mark the bug report 'Fix Released'
or 'Invalid' as appropriate, at the following URL:

  https://bugs.launchpad.net/ubuntu/+bug/BUGNUMBER

"""

message = message_pre_lucid

def summary(bug, bugtask, result):
    fmt         = "%-8s %-10s %-12s %s"
    try:
        print fmt % (bug.id, result, bugtask.status, bug.title)
    except:
        print fmt % (bug.id, result, bugtask.status, "...[unicode error]...")


def main():
    count       = 0

    if dry_run:
        print "DRY RUN - NO CHANGES WILL BE MADE"
    else:
        print "LIVE!  WILL CHANGE BUGS"

    print message
    print "-----------------------------------------------\n"

    s = d.getSourcePackage(name = source_pkg)
    if not s:
        print "Error:  Could not retrieve source package " + source_pkg
        sys.exit(1)
    for bugtask in s.searchTasks(tags = tags, tags_combinator = "All"):
        bug = ArsenalBug(bugtask.bug, lp.launchpad)

        if bug.has_tag("needs-retested-on-lucid-by-june"):
            summary(bug, bugtask, "[ALREADY ASKED]")
            continue
        if bugtask.status == "Incomplete (without response)":
            summary(bug, bugtask, "[UNRESPONDED]")
            continue
        if re_workflow.search(bug.title):
            summary(bug, bugtask, "[WORKFLOW]")
            continue
        if re_master.search(bug.title):
            summary(bug, bugtask, "[MASTER]")
            continue
        if re_needs.search(bug.title):
            summary(bug, bugtask, "[NEEDS SOMETHING]")
            continue
        if bugtask.importance == "Wishlist":
            summary(bug, bugtask, "[WISHLIST]")
            continue
        if not bug.owner_firstname:
            summary(bug, bugtask, "[DUNNO WHO]")
            continue

        m = message.replace("REPORTER", bug.owner_firstname)
        m = m.replace("BUGNUMBER", str(bug.id))

        if not dry_run:
            if bug.append_comment(m):
                if bugtask.status == "Incomplete":
                    bugtask.transitionToStatus(status = "New")
                bugtask.transitionToStatus(status = "Incomplete")
                bug.append_tags("needs-retested-on-lucid-by-june")
        else:
            print m

        summary(bug, bugtask, "-retest-")
        count = count + 1

    if count > 0:
        if dry_run:
            print count, " bugs would be changed for ", source_pkg
        else:
            print count, " bugs changed for ", source_pkg


main()
