#!/usr/bin/python

# Purpose of script is to close all bugs in a given package with
# friendly comment about the change.

from arsenal_lib import *
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: %s <source-package>" %sys.argv[0])
    exit(1)

source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

message_generic = """
This package has become obsolete so we're closing out the bug report as WONTFIX.
Thanks for reporting it though!
"""

message = """
Thank you for reporting this issue about xserver-xorg-video-nv. Starting
with Lucid, Ubuntu is transitioning to using the -nouveau video driver
by default instead of -nv. The reason for this change is because
upstream development for the -nv driver has been quite slow.  We are
quite pleased with the upstream development speed for -nouveau, and hope
this will translate into swifter bug fixes as well.

Because of this, I'm closing this bug report at this time. I'm marking
it wontfix because what you describe is probably a valid issue, but we
do not have further plans to work on it in Ubuntu. If you would still like
to see this issue investigated, I would encourage you to file it
upstream at http://bugs.freedesktop.org/.
"""

for pkg in source_pkgs:
    subject = pkg + " is obsolete"

    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)

        print "Set from " + bugtask.status + " to Won't Fix: " + bug.title
        bug.append_comment(message)
        bugtask.status = "Won't Fix"
        bugtask.lp_save()
