#!/usr/bin/python

from arsenal_lib import *
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package> [source-package ...]\n")
    exit(1)

opt_dryrun  = 0
max_age     = 20
total_count = 0
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

message = """
We're closing this bug since it is has been some time with no response from the original reporter.  However, if the issue still exists please feel free to reopen with the requested information.  Also, if you could, please test against the latest development version of Ubuntu, since this confirms the bug is one we may be able to pass upstream for help.
"""

total = 0
for source_pkg in source_pkgs:
    count = 0

    s = d.getSourcePackage(name = source_pkg)
    for bugtask in s.searchTasks(status="Incomplete (without response)"):
        try:
            bug = ArsenalBug(bugtask.bug, lp.launchpad)

            # Skip if not old enough
            if bug.age_last_message() < max_age:
                continue

            # Skip if bug is a Wishlist
            if bugtask.importance == "Wishlist":
                #print "Skipping wishlist bug",bug.id
                continue

            # Skip if bug is assigned to someone
            if bugtask.assignee:
                #print bug.id,"skipped since assigned to",bugtask.assignee.name
                continue

            # Skip if bug already has an upstream tasks, in which case need to review/expire manually
            if bugtask.bug.bug_watches:
                #print bug.id,"skipped since has upstream bug watches"
                continue

            print bug.id, " ", bugtask.status, " ", bug.title, " [Old bug: ", bug.age_last_message(), " days]"

            if not bug.append_comment(message):
                # Failed to append message (maybe already had it?)
                continue

            new_status = "Expired"
            bugtask.transitionToStatus(status = new_status)
            count = count + 1
            total = total + 1
            print " ---> ",new_status

        except HTTPError, e:
            print "*** Encountered HTTP error processing ", source_pkg, " ***"
            print e.content
            time.sleep(20)
            lp.reset()
            d = lp.load_project("ubuntu")

    if count > 0:
        print count, " bugs changed for ", source_pkg

if total > 0:
    print total, " total bugs changed"
