#!/usr/bin/python

# This script does some basic analysis of bug reports that have
# reached the Confirmed state to improve them for reviewing,
# filter out bad ones, extract useful information into the description,
# and other actions to prepare them for upstreaming or resolving.

from arsenal_lib import *

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = 0
total_count = 0
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

total_count = 0

try:
    for source_pkg in source_pkgs:
        count=0
        print "Processing ", source_pkg
        for bugtask in d.searchTasks(search_text=source_pkg, status="Confirmed"):
            if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
                continue

            bug = ArsenalBug(bugtask.bug, lp.launchpad)

            print bug.id, "  ", bugtask.status, "  ", bug.title

            # TODO:
            #   * Are there any needs-* tags?
            #     - If so, and if the requested info is now present, remove the tags
            #     - else, if no comment since the request, reset to Incomplete
            #     - else, if all commenters since request are not the reporter or a triager
            #       and none have more than 100 karma, reset to Incomplete
            #   * Has a recent comment mentioned a trigger phrase, like 'fixed', 'works now',
            #     'cannot reproduce', 'no access to hardware', etc.?
            #   * Verify attachments marked patch is indeed a patch
            #   * Is there an upstream task with a patch on it?
            #     - If so, if the patch isn't already attached, attach it here
            #   * Xorg.0.log processing
            #     - Extract all error and warning messages
            #       + Omit ones that are known to be harmless
            #     - If resolution-related, extract the EDID/DDC sections
            #     - If has a backtrace, extract it and put in description if no backtrace there yet
            #   * If crash bug, does it have a useful backtrace?
            #     - If not, reset to Incomplete and ask for one
            #     - Tag for re-tracer if trace is missing symbols
            #     - If so, extract trace and put into description
            #     - Set to High importance
            #     - If private but no core file, mark it un-private
            #   * If appears to be an SRU request, add template items to description
            #   * If an -intel quirk bug, attempt to construct the quirk code
            #   * Try to identify bug's family, set importance, suggest triaging steps,
            #     and tag bitesize/mentoring where appropriate
            #     - wrong resolution due to bad edid -> quirks
            #     - wrong resolution due to output issues -> quirks
            #     - AGP issues on -ati
            #     - EQ overflow
            #     - crashes with full backtraces
            #     - lid close w/ intel
            #     - font dpi issues
            #     - black/blank screen bugs
            #     - high cpu usage bugs
            #     - screen corruption
            #     - fails to start
            #     - fails to login
            #     - fails to detect
            #     - no modes
            #     - fglrx/ati installation incompatibilities
            #     - 3D/compiz/opengl issues (may be mesa...)
            #     - If tagged as a regression, perhaps referring to X/Bisecting

            # TODO:  Update bug
            if 1 == 2:
                count += 1
                total_count += 1

                new_status = "Triaged"
                try:
                    bugtask.transitionToStatus(status = new_status)
                    print " ---> " + new_status
                except:
                    print " ERROR:  Failed to update bug ", bug.id
                    print "New status was:  ", new_status
                    raise

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
