#!/usr/bin/python

# This script is meant to be periodically run against all of the X.org
# bugs to modify titles to add details, fix problems, and clarify.
# This is intended to help simplify forwarding the bugs upstream.

from arsenal_lib import *
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = True

def cleanup_intel(title, pci_device):
    re_bracket = re.compile("\[.*\]+")
    if re_bracket.search(title):
        # Appears to already have been nametagged
        return title

    re_number = re.compile("^\[[89]\d")
    if re_number.search(title):
        return "[i" + title[1:]

    # Fix iGxx
    re_igxx = re.compile("^\[i(gm*[34]\d.*)", re.IGNORECASE)
    m = re_igxx.search(title)
    if m:
        return "[" + m.group(1)

    re_NN5 = re.compile("([89][456]5)")
    m = re_NN5.search(title)
    if m:
        return "[i" + m.group(1) + "] " + title

    re_830 = re.compile("82830")
    if re_830.search(title):
        return "[i830] " + title

    re_g45 = re.compile("(gm*45|4500MHD)", re.IGNORECASE)
    if re_g45.search(title):
        return "[g45] " + title

    re_g35 = re.compile("gm*35", re.IGNORECASE)
    if re_g35.search(title):
        return "[g35] " + title

    # Else, use the pci_device to get the identifier
    if pci_device:
        return "[%s] %s" % (pci_device, title)
    
    return title

def cleanup_ati(title, pci_device):
    re_bracket = re.compile("\[.*\]+")
    if re_bracket.search(title):
        # Appears to already have been nametagged
        return title

    if pci_device:
        return "[%s] %s" % (pci_device, title)

    return title

pci_devices = [
    # Intel
    { 'name':'i810',        're':'(8086:7121)' },
    { 'name':'i810dc',      're':'(8086:7123)' },
    { 'name':'i810e',       're':'(8086:7125)' },
    { 'name':'i815',        're':'(8086:1132|82815)' },
    { 'name':'i830',        're':'(8086:3577|82830)' },
    { 'name':'i845',        're':'(8086:2562|82845G)' },
    { 'name':'i855',        're':'(8086:3582|855GM)' },
    { 'name':'i865',        're':'(8086:2572|82865G)' },
    { 'name':'i915g',       're':'(8086:2582)' },
    { 'name':'i915gm',      're':'(8086:2592|915GM)' },
    { 'name':'e7221',       're':'(8086:258a)' },
    { 'name':'i945g',       're':'(8086:2772|945G[ \/]|82945G[ \/])' },
    { 'name':'i945gm',      're':'(8086:27a2|945GM[ \/]|82945GM[ \/])' },
    { 'name':'i945gme',     're':'(8086:27ae|945GME|82945GME)' },
    { 'name':'IGDg',        're':'(8086:a001)' },
    { 'name':'IGDgm',       're':'(8086:a011)' },
    { 'name':'pineviewg',   're':'(8086:a001)' },
    { 'name':'pineviewgm',  're':'(8086:a011)' },
    { 'name':'i946gz',      're':'(8086:2972|82946GZ)' },
    { 'name':'g35',         're':'(8086:2982|82G35)' },
    { 'name':'i965q',       're':'(8086:2992|Q965)' },
    { 'name':'i965g',       're':'(8086:29a2|G965)' },
    { 'name':'g33',         're':'(8086:29c2|82G33)' },
    { 'name':'q35',         're':'(8086:29b2)' },
    { 'name':'q33',         're':'(8086:29d2)' },
    { 'name':'i965gm',      're':'(8086:2a02|GM965)' },
    { 'name':'i965gme',     're':'(8086:2a12)' },
    { 'name':'gm45',        're':'(8086:2a42)' },
    { 'name':'IGDeg',       're':'(8086:2e02)' },
    { 'name':'q45',         're':'(8086:2e12)' },
    { 'name':'g45',         're':'(8086:2e22)' },
    { 'name':'g41',         're':'(8086:2e32)' },
    { 'name':'b43',         're':'(8086:2e42)' },
    { 'name':'clarkdale',   're':'(8086:0042)' },
    { 'name':'arrandale',   're':'(8086:0046)' },
    { 'name':'sandybridge', 're':'(8086:0102)' },
    { 'name':'sandybridgem','re':'(8086:0106)' },

    # ATI
    { 'name':'RV380',       're':'(1002:3150)' },
    { 'name':'RV380',       're':'(1002:3151)' },
    { 'name':'RV380',       're':'(1002:3152)' },
    { 'name':'RV380',       're':'(1002:3154)' },
    { 'name':'RV380',       're':'(1002:3e50)' },
    { 'name':'RV380',       're':'(1002:3e54)' },
    { 'name':'RS100',       're':'(1002:4136)' },
    { 'name':'RS200',       're':'(1002:4137)' },
    { 'name':'R300',        're':'(1002:4144)' },
    { 'name':'R300',        're':'(1002:4145)' },
    { 'name':'R300',        're':'(1002:4146)' },
    { 'name':'R300',        're':'(1002:4147)' },
    { 'name':'R350',        're':'(1002:4148)' },
    { 'name':'R350',        're':'(1002:4149)' },
    { 'name':'R350',        're':'(1002:414a)' },
    { 'name':'R350',        're':'(1002:414b)' },
    { 'name':'RV350',       're':'(1002:4150)' },
    { 'name':'RV350',       're':'(1002:4151)' },
    { 'name':'RV360',       're':'(1002:4152)' },
    { 'name':'RV350',       're':'(1002:4153)' },
    { 'name':'RV350',       're':'(1002:4154)' },
    { 'name':'RV350',       're':'(1002:4155)' },
    { 'name':'RV350',       're':'(1002:4156)' },
    { 'name':'RS250',       're':'(1002:4237)' },
    { 'name':'R200',        're':'(1002:4242)' },
    { 'name':'R200',        're':'(1002:4243)' },
    { 'name':'RS100',       're':'(1002:4336)' },
    { 'name':'RS200',       're':'(1002:4337)' },
    { 'name':'RS250',       're':'(1002:4437)' },
    { 'name':'RV250',       're':'(1002:4966)' },
    { 'name':'RV250',       're':'(1002:4967)' },
    { 'name':'R420',        're':'(1002:4a48)' },
    { 'name':'R420',        're':'(1002:4a49)' },
    { 'name':'R420',        're':'(1002:4a4a)' },
    { 'name':'R420',        're':'(1002:4a4b)' },
    { 'name':'R420',        're':'(1002:4a4c)' },
    { 'name':'R420',        're':'(1002:4a4d)' },
    { 'name':'R420',        're':'(1002:4a4e)' },
    { 'name':'R420',        're':'(1002:4a4f)' },
    { 'name':'R481',        're':'(1002:4b48)' },
    { 'name':'R481',        're':'(1002:4b49)' },
    { 'name':'R481',        're':'(1002:4a4a)' },
    { 'name':'R481',        're':'(1002:4a4b)' },
    { 'name':'R481',        're':'(1002:4a4c)' },
    { 'name':'RV200',       're':'(1002:4c57)' },
    { 'name':'RV200',       're':'(1002:4c58)' },
    { 'name':'RV100',       're':'(1002:4c59)' },
    { 'name':'RV100',       're':'(1002:4c5a)' },
    { 'name':'RV250',       're':'(1002:4c64)' },
    { 'name':'RV250',       're':'(1002:4c66)' },
    { 'name':'RV250',       're':'(1002:4c67)' },
    { 'name':'R300',        're':'(1002:4e44)' },
    { 'name':'R300',        're':'(1002:4e45)' },
    { 'name':'R300',        're':'(1002:4e46)' },
    { 'name':'R300',        're':'(1002:4e47)' },
    { 'name':'R350',        're':'(1002:4e48)' },
    { 'name':'R350',        're':'(1002:4e49)' },
    { 'name':'R360',        're':'(1002:4e4a)' },
    { 'name':'R350',        're':'(1002:4e4b)' },
    { 'name':'RV350',       're':'(1002:4e50)' },
    { 'name':'RV350',       're':'(1002:4e51)' },
    { 'name':'RV350',       're':'(1002:4e52)' },
    { 'name':'RV350',       're':'(1002:4e53)' },
    { 'name':'RV350',       're':'(1002:4e54)' },
    { 'name':'RV350',       're':'(1002:4e56)' },
    { 'name':'R200',        're':'(1002:5148)' },
    { 'name':'R200',        're':'(1002:514c)' },
    { 'name':'R200',        're':'(1002:514d)' },
    { 'name':'RV200',       're':'(1002:5157)' },
    { 'name':'RV200',       're':'(1002:5158)' },
    { 'name':'RV100',       're':'(1002:5159)' },
    { 'name':'RV100',       're':'(1002:515a)' },
    { 'name':'RN50',        're':'(1002:515e)' },
    { 'name':'RV370',       're':'(1002:5160)' },
    { 'name':'RV370',       're':'(1002:5162)' },
    { 'name':'RV370',       're':'(1002:5164)' },
    { 'name':'RV380',       're':'(1002:5460)' },
    { 'name':'RV380',       're':'(1002:5462)' },
    { 'name':'RV380',       're':'(1002:5464)' },
    { 'name':'R423',        're':'(1002:5548)' },
    { 'name':'R423',        're':'(1002:5549)' },
    { 'name':'R423',        're':'(1002:554a)' },
    { 'name':'R423',        're':'(1002:554b)' },
    { 'name':'R430',        're':'(1002:554c)' },
    { 'name':'R430',        're':'(1002:554d)' },
    { 'name':'R430',        're':'(1002:554e)' },
    { 'name':'R430',        're':'(1002:554f)' },
    { 'name':'R423',        're':'(1002:5550)' },
    { 'name':'R423',        're':'(1002:5551)' },
    { 'name':'R423',        're':'(1002:5552)' },
    { 'name':'R423',        're':'(1002:5554)' },
    { 'name':'RV410',       're':'(1002:564a)' },
    { 'name':'RV410',       're':'(1002:564b)' },
    { 'name':'RV410',       're':'(1002:5652)' },
    { 'name':'RV410',       're':'(1002:5653)' },
    { 'name':'RV410',       're':'(1002:5657)' },
    { 'name':'RS300',       're':'(1002:5834)' },
    { 'name':'RS300',       're':'(1002:5835)' },
    { 'name':'RS480',       're':'(1002:5954)' },
    { 'name':'RS480',       're':'(1002:5955)' },
    { 'name':'RV280',       're':'(1002:5960)' },
    { 'name':'RV280',       're':'(1002:5961)' },
    { 'name':'RV280',       're':'(1002:5962)' },
    { 'name':'RV280',       're':'(1002:5964)' },
    { 'name':'RV280',       're':'(1002:5965)' },
    { 'name':'RN50',        're':'(1002:5969)' },
    { 'name':'RS482',       're':'(1002:5974)' },
    { 'name':'RS485',       're':'(1002:5975)' },
    { 'name':'RS400',       're':'(1002:5a41)' },
    { 'name':'RS400',       're':'(1002:5a42)' },
    { 'name':'RC410',       're':'(1002:5a61)' },
    { 'name':'RC410',       're':'(1002:5a62)' },
    { 'name':'RV370',       're':'(1002:5b60)' },
    { 'name':'RV370',       're':'(1002:5b62)' },
    { 'name':'RV370',       're':'(1002:5b63)' },
    { 'name':'RV370',       're':'(1002:5b64)' },
    { 'name':'RV370',       're':'(1002:5b65)' },
    { 'name':'RV280',       're':'(1002:5c61)' },
    { 'name':'RV280',       're':'(1002:5c63)' },
    { 'name':'R430',        're':'(1002:5d48)' },
    { 'name':'R430',        're':'(1002:5d49)' },
    { 'name':'R430',        're':'(1002:5d4a)' },
    { 'name':'R480',        're':'(1002:5d4c)' },
    { 'name':'R480',        're':'(1002:5d4d)' },
    { 'name':'R480',        're':'(1002:5d4e)' },
    { 'name':'R480',        're':'(1002:5d4f)' },
    { 'name':'R480',        're':'(1002:5d50)' },
    { 'name':'R480',        're':'(1002:5d52)' },
    { 'name':'R423',        're':'(1002:5d57)' },
    { 'name':'RV410',       're':'(1002:5e48)' },
    { 'name':'RV410',       're':'(1002:5e4a)' },
    { 'name':'RV410',       're':'(1002:5e4b)' },
    { 'name':'RV410',       're':'(1002:5e4c)' },
    { 'name':'RV410',       're':'(1002:5e4d)' },
    { 'name':'RV410',       're':'(1002:5e4f)' },
    { 'name':'R520',        're':'(1002:7100)' },
    { 'name':'R520',        're':'(1002:7101)' },
    { 'name':'R520',        're':'(1002:7102)' },
    { 'name':'R520',        're':'(1002:7103)' },
    { 'name':'R520',        're':'(1002:7104)' },
    { 'name':'R520',        're':'(1002:7105)' },
    { 'name':'R520',        're':'(1002:7106)' },
    { 'name':'R520',        're':'(1002:7108)' },
    { 'name':'R520',        're':'(1002:7109)' },
    { 'name':'R520',        're':'(1002:710a)' },
    { 'name':'R520',        're':'(1002:710b)' },
    { 'name':'R520',        're':'(1002:710c)' },
    { 'name':'R520',        're':'(1002:710e)' },
    { 'name':'R520',        're':'(1002:710f)' },
    { 'name':'RV515',       're':'(1002:7140)' },
    { 'name':'RV515',       're':'(1002:7141)' },
    { 'name':'RV515',       're':'(1002:7142)' },
    { 'name':'RV515',       're':'(1002:7143)' },
    { 'name':'RV515',       're':'(1002:7144)' },
    { 'name':'RV515',       're':'(1002:7145)' },
    { 'name':'RV515',       're':'(1002:7146)' },
    { 'name':'RV515',       're':'(1002:7147)' },
    { 'name':'RV515',       're':'(1002:7149)' },
    { 'name':'RV515',       're':'(1002:714a)' },
    { 'name':'RV515',       're':'(1002:714b)' },
    { 'name':'RV515',       're':'(1002:714c)' },
    { 'name':'RV515',       're':'(1002:714d)' },
    { 'name':'RV515',       're':'(1002:714e)' },
    { 'name':'RV515',       're':'(1002:714f)' },
    { 'name':'RV515',       're':'(1002:7151)' },
    { 'name':'RV515',       're':'(1002:7152)' },
    { 'name':'RV515',       're':'(1002:7153)' },
    { 'name':'RV515',       're':'(1002:715e)' },
    { 'name':'RV515',       're':'(1002:715f)' },
    { 'name':'RV515',       're':'(1002:7180)' },
    { 'name':'RV515',       're':'(1002:7181)' },
    { 'name':'RV515',       're':'(1002:7183)' },
    { 'name':'RV515',       're':'(1002:7186)' },
    { 'name':'RV515',       're':'(1002:7187)' },
    { 'name':'RV515',       're':'(1002:7188)' },
    { 'name':'RV515',       're':'(1002:718a)' },
    { 'name':'RV515',       're':'(1002:718b)' },
    { 'name':'RV515',       're':'(1002:718c)' },
    { 'name':'RV515',       're':'(1002:718d)' },
    { 'name':'RV515',       're':'(1002:718f)' },
    { 'name':'RV515',       're':'(1002:7193)' },
    { 'name':'RV515',       're':'(1002:7196)' },
    { 'name':'RV515',       're':'(1002:719b)' },
    { 'name':'RV515',       're':'(1002:719f)' },
    { 'name':'RV530',       're':'(1002:71c0)' },
    { 'name':'RV530',       're':'(1002:71c1)' },
    { 'name':'RV530',       're':'(1002:71c2)' },
    { 'name':'RV530',       're':'(1002:71c3)' },
    { 'name':'RV530',       're':'(1002:71c4)' },
    { 'name':'RV530',       're':'(1002:71c5)' },
    { 'name':'RV530',       're':'(1002:71c6)' },
    { 'name':'RV530',       're':'(1002:71c7)' },
    { 'name':'RV530',       're':'(1002:71cd)' },
    { 'name':'RV530',       're':'(1002:71ce)' },
    { 'name':'RV530',       're':'(1002:71d2)' },
    { 'name':'RV530',       're':'(1002:71d4)' },
    { 'name':'RV530',       're':'(1002:71d5)' },
    { 'name':'RV530',       're':'(1002:71d6)' },
    { 'name':'RV530',       're':'(1002:71da)' },
    { 'name':'RV530',       're':'(1002:71de)' },
    { 'name':'RV515',       're':'(1002:7200)' },
    { 'name':'RV515',       're':'(1002:7210)' },
    { 'name':'RV515',       're':'(1002:7211)' },
    { 'name':'R580',        're':'(1002:7240)' },
    { 'name':'R580',        're':'(1002:7243)' },
    { 'name':'R580',        're':'(1002:7244)' },
    { 'name':'R580',        're':'(1002:7245)' },
    { 'name':'R580',        're':'(1002:7246)' },
    { 'name':'R580',        're':'(1002:7247)' },
    { 'name':'R580',        're':'(1002:7248)' },
    { 'name':'R580',        're':'(1002:7249)' },
    { 'name':'R580',        're':'(1002:724a)' },
    { 'name':'R580',        're':'(1002:724b)' },
    { 'name':'R580',        're':'(1002:724c)' },
    { 'name':'R580',        're':'(1002:724d)' },
    { 'name':'R580',        're':'(1002:724e)' },
    { 'name':'R580',        're':'(1002:724f)' },
    { 'name':'RV570',       're':'(1002:7280)' },
    { 'name':'RV560',       're':'(1002:7281)' },
    { 'name':'RV560',       're':'(1002:7283)' },
    { 'name':'R580',        're':'(1002:7284)' },
    { 'name':'RV560',       're':'(1002:7287)' },
    { 'name':'RV570',       're':'(1002:7288)' },
    { 'name':'RV570',       're':'(1002:7289)' },
    { 'name':'RV570',       're':'(1002:728b)' },
    { 'name':'RV570',       're':'(1002:728c)' },
    { 'name':'RV560',       're':'(1002:7290)' },
    { 'name':'RV560',       're':'(1002:7291)' },
    { 'name':'RV560',       're':'(1002:7293)' },
    { 'name':'RV560',       're':'(1002:7297)' },
    { 'name':'RS350',       're':'(1002:7834)' },
    { 'name':'RS350',       're':'(1002:7835)' },
    { 'name':'RS690',       're':'(1002:791e)' },
    { 'name':'RS690',       're':'(1002:791f)' },
    { 'name':'RS600',       're':'(1002:793f)' },
    { 'name':'RS600',       're':'(1002:7941)' },
    { 'name':'RS600',       're':'(1002:7942)' },
    { 'name':'RS740',       're':'(1002:796c)' },
    { 'name':'RS740',       're':'(1002:796d)' },
    { 'name':'RS740',       're':'(1002:796e)' },
    { 'name':'RS740',       're':'(1002:796f)' },
    { 'name':'R600',        're':'(1002:9400)' },
    { 'name':'R600',        're':'(1002:9401)' },
    { 'name':'R600',        're':'(1002:9402)' },
    { 'name':'R600',        're':'(1002:9403)' },
    { 'name':'R600',        're':'(1002:9405)' },
    { 'name':'R600',        're':'(1002:940a)' },
    { 'name':'R600',        're':'(1002:940b)' },
    { 'name':'R600',        're':'(1002:940f)' },
    { 'name':'RV770',       're':'(1002:9440)' },
    { 'name':'RV770',       're':'(1002:9441)' },
    { 'name':'RV770',       're':'(1002:9442)' },
    { 'name':'RV770',       're':'(1002:9443)' },
    { 'name':'RV770',       're':'(1002:9444)' },
    { 'name':'RV770',       're':'(1002:9446)' },
    { 'name':'RV770',       're':'(1002:944a)' },
    { 'name':'RV770',       're':'(1002:944b)' },
    { 'name':'RV770',       're':'(1002:944c)' },
    { 'name':'RV770',       're':'(1002:944e)' },
    { 'name':'RV770',       're':'(1002:9450)' },
    { 'name':'RV770',       're':'(1002:9452)' },
    { 'name':'RV770',       're':'(1002:9456)' },
    { 'name':'RV770',       're':'(1002:945a)' },
    { 'name':'RV770',       're':'(1002:945b)' },
    { 'name':'RV790',       're':'(1002:9460)' },
    { 'name':'RV790',       're':'(1002:9462)' },
    { 'name':'RV770',       're':'(1002:946a)' },
    { 'name':'RV770',       're':'(1002:946b)' },
    { 'name':'RV770',       're':'(1002:947a)' },
    { 'name':'RV770',       're':'(1002:947b)' },
    { 'name':'RV730',       're':'(1002:9480)' },
    { 'name':'RV730',       're':'(1002:9487)' },
    { 'name':'RV730',       're':'(1002:9488)' },
    { 'name':'RV730',       're':'(1002:9489)' },
    { 'name':'RV730',       're':'(1002:948f)' },
    { 'name':'RV730',       're':'(1002:9490)' },
    { 'name':'RV730',       're':'(1002:9491)' },
    { 'name':'RV730',       're':'(1002:9495)' },
    { 'name':'RV730',       're':'(1002:9498)' },
    { 'name':'RV730',       're':'(1002:949c)' },
    { 'name':'RV730',       're':'(1002:949e)' },
    { 'name':'RV730',       're':'(1002:949f)' },
    { 'name':'RV740',       're':'(1002:94a0)' },
    { 'name':'RV740',       're':'(1002:94a1)' },
    { 'name':'RV740',       're':'(1002:94a3)' },
    { 'name':'RV740',       're':'(1002:94b1)' },
    { 'name':'RV740',       're':'(1002:94b3)' },
    { 'name':'RV740',       're':'(1002:94b4)' },
    { 'name':'RV740',       're':'(1002:94b5)' },
    { 'name':'RV740',       're':'(1002:94b9)' },
    { 'name':'RV610',       're':'(1002:94c0)' },
    { 'name':'RV610',       're':'(1002:94c1)' },
    { 'name':'RV610',       're':'(1002:94c3)' },
    { 'name':'RV610',       're':'(1002:94c4)' },
    { 'name':'RV610',       're':'(1002:94c5)' },
    { 'name':'RV610',       're':'(1002:94c6)' },
    { 'name':'RV610',       're':'(1002:94c7)' },
    { 'name':'RV610',       're':'(1002:94c8)' },
    { 'name':'RV610',       're':'(1002:94c9)' },
    { 'name':'RV610',       're':'(1002:94cb)' },
    { 'name':'RV610',       're':'(1002:94cc)' },
    { 'name':'RV610',       're':'(1002:94cd)' },
    { 'name':'RV670',       're':'(1002:9500)' },
    { 'name':'RV670',       're':'(1002:9501)' },
    { 'name':'RV670',       're':'(1002:9504)' },
    { 'name':'RV670',       're':'(1002:9505)' },
    { 'name':'RV670',       're':'(1002:9506)' },
    { 'name':'RV670',       're':'(1002:9507)' },
    { 'name':'RV670',       're':'(1002:9508)' },
    { 'name':'RV670',       're':'(1002:9509)' },
    { 'name':'RV670',       're':'(1002:950f)' },
    { 'name':'RV670',       're':'(1002:9511)' },
    { 'name':'RV670',       're':'(1002:9515)' },
    { 'name':'RV670',       're':'(1002:9517)' },
    { 'name':'RV670',       're':'(1002:9519)' },
    { 'name':'RV710',       're':'(1002:9540)' },
    { 'name':'RV710',       're':'(1002:9541)' },
    { 'name':'RV710',       're':'(1002:954e)' },
    { 'name':'RV710',       're':'(1002:954f)' },
    { 'name':'RV710',       're':'(1002:9552)' },
    { 'name':'RV710',       're':'(1002:9553)' },
    { 'name':'RV710',       're':'(1002:9555)' },
    { 'name':'RV710',       're':'(1002:9557)' },
    { 'name':'RV630',       're':'(1002:9580)' },
    { 'name':'RV630',       're':'(1002:9581)' },
    { 'name':'RV630',       're':'(1002:9583)' },
    { 'name':'RV630',       're':'(1002:9586)' },
    { 'name':'RV630',       're':'(1002:9587)' },
    { 'name':'RV630',       're':'(1002:9588)' },
    { 'name':'RV630',       're':'(1002:9589)' },
    { 'name':'RV630',       're':'(1002:958a)' },
    { 'name':'RV630',       're':'(1002:958b)' },
    { 'name':'RV630',       're':'(1002:958c)' },
    { 'name':'RV630',       're':'(1002:958d)' },
    { 'name':'RV630',       're':'(1002:958e)' },
    { 'name':'RV630',       're':'(1002:958f)' },
    { 'name':'RV710',       're':'(1002:9542)' },
    { 'name':'RV620',       're':'(1002:95c0)' },
    { 'name':'RV620',       're':'(1002:95c2)' },
    { 'name':'RV620',       're':'(1002:95c4)' },
    { 'name':'RV620',       're':'(1002:95c5)' },
    { 'name':'RV620',       're':'(1002:95c6)' },
    { 'name':'RV620',       're':'(1002:95c7)' },
    { 'name':'RV620',       're':'(1002:95c9)' },
    { 'name':'RV620',       're':'(1002:95cc)' },
    { 'name':'RV620',       're':'(1002:95cd)' },
    { 'name':'RV620',       're':'(1002:95ce)' },
    { 'name':'RV620',       're':'(1002:95cf)' },
    { 'name':'RV635',       're':'(1002:9590)' },
    { 'name':'RV635',       're':'(1002:9596)' },
    { 'name':'RV635',       're':'(1002:9597)' },
    { 'name':'RV635',       're':'(1002:9598)' },
    { 'name':'RV635',       're':'(1002:9599)' },
    { 'name':'RV635',       're':'(1002:9591)' },
    { 'name':'RV635',       're':'(1002:9593)' },
    { 'name':'RV635',       're':'(1002:9595)' },
    { 'name':'RV635',       're':'(1002:959b)' },
    { 'name':'RS780',       're':'(1002:9610)' },
    { 'name':'RS780',       're':'(1002:9611)' },
    { 'name':'RS780',       're':'(1002:9612)' },
    { 'name':'RS780',       're':'(1002:9613)' },
    { 'name':'RS780',       're':'(1002:9614)' },
    { 'name':'RS780',       're':'(1002:9615)' },
    { 'name':'RS780',       're':'(1002:9616)' },
    { 'name':'RS880',       're':'(1002:9710)' },
    { 'name':'RS880',       're':'(1002:9711)' },
    { 'name':'RS880',       're':'(1002:9712)' },
    { 'name':'RS880',       're':'(1002:9713)' },
    { 'name':'RS880',       're':'(1002:9714)' },
    { 'name':'RS880',       're':'(1002:9715)' },

    # Nvidia
    { 'name':'G84 8600M GT','re':'(10de:0407)' },
    ]
for device in pci_devices:
    device['rc'] = re.compile(device['re'], re.IGNORECASE)

def get_pci_device(desc):
    regex_vga = re.compile('VGA compatible controller (.*)', re.IGNORECASE)

    lines = regex_vga.findall(desc)
    if len(lines) > 0:
        for l in lines:
            if len(l.strip())>0:
                for device in pci_devices:
                    if device['rc'].search(l.strip()):
                        return device['name']
                print "Unknown device: '%s'" %(l.strip())
    
    return None

spellcheck_regexes = [
    {'re':'crach',                    'fix':'crash'},
    {'re':'du[ae]l *(head|display)',  'fix':'dual monitor'},
    {'re':'duel monitor',             'fix':'dual monitor'},
    {'re':'kayboard',                 'fix':'keyboard'},
    {'re':'start up',                 'fix':'startup'},
    {'re':' X ',                      'fix':' Xorg '},
    {'re':' (res|rez) ',              'fix':' resolution '},
    {'re':' freezed ',                'fix':' freezes '},
    {'re':' (hangs|locks *up) ',      'fix':' freezes '},
    {'re':' eating ',                 'fix':' using '},
    {'re':' upon ',                   'fix':' on '},
    {'re':' (garbled|scrambled) ',    'fix':' corrupted '},
    {'re':' strange graphics ',       'fix':' corruption '},
    {'re':' glitches ',               'fix':' corruption '},
    {'re':' sluggish ',               'fix':' slow '},
    {'re':' sluggishness ',           'fix':' slowness '},
    {'re':'artefact',                 'fix':'artifact'},
    {'re':'malfunctions',             'fix':'does not work'},
    {'re':'nonfunctional',            'fix':'does not work'},
    {'re':'(weird|wrong|craz+y|awful|borked|messed-up|unusable|broken)', 'fix':'incorrect'},
    {'re':'unreadable',               'fix':'incorrectly displayed'},
    {'re':'does not startup',         'fix':'fails to start'},
    {'re':'does not load at startup', 'fix':'fails to start'},
    {'re':'freezes after logging in', 'fix':'fails to login'},
    {'re':'screen illumination',      'fix':'backlight'},
    {'re':'gdm restart',              'fix':'Xorg restart'},
    {'re':'segmentation fault',       'fix':'SIGSEGV'},
    {'re':'segfault',                 'fix':'SIGSEGV'},
    {'re':'out of scan range',        'fix':'out of range'},
    {'re':'interpid',                 'fix':'intrepid'},
    ]

def spellfix(title):
    for regex in spellcheck_regexes:
        if not 'rc' in regex:
            regex['rc'] = re.compile(regex['re'])
        title = regex['rc'].sub(regex['fix'], title)
    return title
        

total_renamed = 0
for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        text = "%-8d  %-12s  %s" % (bug.id, bugtask.status, bug.title)

        pci_device = get_pci_device(bug.description)
        title = bug.title
        fixed_title = spellfix(bug.title)
        if fixed_title != title:
            print "Would like to cleanup %s to %s" %(title, fixed_title)

        if pkg == "xserver-xorg-video-intel":
            title = cleanup_intel(title, pci_device)

        if pkg in ["fglrx-installer", "xserver-xorg-video-ati"]:
            title = cleanup_ati(title, pci_device)

        if title and title != bug.title:
            if not dryrun:
                bug.bug.title = title
                bug.bug.lp_save()
            print text
            print "  ---------> Renamed to %s" % (title)
            total_renamed += 1
        else:
            print text
            if not pci_device:
                print "  ---------> Couldn't figure out pci device for this bug"
            else:
                print "  ---------> Fine as is"

if total_renamed > 0:
    print "Total renamed: ",total_renamed

