#!/usr/bin/python

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
total_count = 0
tagged      = 0
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = False

tags_regexes = [
    {'name':'kubuntu',   're':'(kubuntu|kdm|kwin|\bkde|konqueror|konsole)', },
    {'name':'xubuntu',   're':'xubuntu', },
    {'name':'ppc',       're':'(powerpc|[\[\b]ppc[\[\b])', },
    {'name':'edgers',    're':'xorg-edgers', },
]
for tag_regex in tags_regexes:
    tag_regex['rc'] = re.compile(tag_regex['re'], re.IGNORECASE)

for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        if bug.ubuntu_omittable():
            continue

        total_count += 1
        bugline = "%-7s %-11s %s" % (bug.id, bugtask.status, bug.title.decode("utf-8").encode("ascii", "replace"))

        if bugtask.importance == "Wishlist":
            print bugline
            print "  --> Wishlist"
            continue

        desc = bug.description.decode("utf-8")

        # Search the title and description for tags
        for tag in tags_regexes:
            if tag['name'] in bug.bug.tags:
                print bugline
                print "  ----> Already tagged ",tag['name']
                continue

            if tag['rc'].search(bug.title):
                print bugline
                print "  --> title: ",tag['name']
                if not dryrun:
                    bug.append_tags(tag['name'])
                tagged += 1
                continue

            if tag['rc'].search(desc):
                print bugline
                print "  --> desc: ",tag['name']
                if not dryrun:
                    bug.append_tags(tag['name'])
                tagged += 1
                continue

        # Search for the distro release version
        releases = determine_distros(bug)
        if len(releases)>0:
            if dryrun:
                print bugline
                print " --> releases: "," ".join(releases)
            elif bug.append_tags(releases):
                print bugline
                print " --> releases: "," ".join(releases)

print total_count," total bugs, ",tagged," tagged"

