#!/usr/bin/python

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltk import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
total_count = 0
tagged      = 0
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
opt_dryrun  = False

tags_regexes = [
    {'name':'freeze',        're':'(freeze|freezing| hangs* |locks* *up|system unresponsive)', },
    {'name':'error-exit',    're':'(exits with|server terminated|kills session)', },
    {'name':'error-exit',    're':'(forces a logout)', },
    {'name':'no-screens',    're':'(no screens found|screen not detected)', },
    {'name':'crash',         're':'(crash|crasch|gdm_slave_xioerror_handler)', },
    {'name':'corruption',    're':'(corrupt|rubbish|garbage|garbled|display bugs|colored noise)', },
    {'name':'tearing',       're':'(tearing|videos.* tear)', },
    {'name':'resolution',    're':'(resolution| mode )', },
    {'name':'performance',   're':'(performance|sluggish|jerky|slow at redraw)', },
    {'name':'flicker',       're':'(flicker|blinking|blinks|flashes periodically)', },
    {'name':'dual-head',     're':'(dual|ext|external)[ -]*(head|monitor)', },
    {'name':'backlight',     're':'(backlight|brightness|display could not be turned on)', },
    {'name':'videoplayback', 're':'( Xv |mplayer|xine|totem)', },
    {'name':'ghost-monitor', 're':'(false|non-existant) (monitor|vga )', },
    {'name':'3d',            're':'(3D |OpenGL| GL |Blender)', },
    {'name':'compiz',        're':'( compiz|desktop effects)', },
    {'name':'resume',        're':'(suspend|resume|hibernate|wake up)', },
    {'name':'cropped',       're':'(cropped)', },
    {'name':'font-size',     're':'(text .* too large|huge font)', },
    {'name':'black-screen',  're':'(black|blank) (or white |)(tv |video|screen)', },
    {'name':'black-screen',  're':'(display|screen) (goes|stays|went|displays).* (black|blank|nothing)', },
    {'name':'vt-switch',     're':'(VT switch|ctrl-alt-f|tty switch|switch .*to console)', },
    {'name':'tv-out',        're':'(TV[ -]*Out|S[ -]*Video| HDMI )', },
    {'name':'high-cpu',      're':'(high \%*cpu|cpu \d+\%|keeps processor.*busy|high load)', },
    {'name':'mouse-pointer', 're':'(mouse pointer)', },
]
for tag_regex in tags_regexes:
    tag_regex['rc'] = re.compile(tag_regex['re'], re.IGNORECASE)

regex_kubuntu = re.compile("(kubuntu| kde| kwin)", re.IGNORECASE)
regex_xubuntu = re.compile("(xubuntu)", re.IGNORECASE)


# TODO:  Enable this to apply to more than just -intel
for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        if bug.ubuntu_omittable():
            continue

        total_count += 1
        title = bug.title.decode("utf-8")
        desc = bug.description.decode("utf-8")
        text = "%-7s %-11s %s" % (bug.id, bugtask.status, title)

        # Search description for key tags
        if regex_kubuntu.search(title) or regex_kubuntu.search(desc):
            if not opt_dryrun:
                bug.append_tags("kubuntu")
            print text
            print "  --> Tag kubuntu"
            tagged += 1
        elif regex_xubuntu.search(title) or regex_xubuntu.search(desc):
            if not opt_dryrun:
                bug.append_tags("xubuntu")
            print "  --> Tag xubuntu"
            print text
            tagged += 1

        # If we found the driver listed in Xorg.0.log, transition to it
        for tag in tags_regexes:
            if tag['name'] in bug.bug.tags:
                continue

            if tag['rc'].search(title):
                if not opt_dryrun:
                    bug.append_tags(tag['name'])
                print text
                print "  --> Tag %s" %(tag['name'])
                tagged += 1

if total_count > 0:
    print total_count," total bugs, ",tagged," tagged"

